@@ -1,19 +1,22 @@
-#!/usr/bin/env python
-
 # Part of the psychopy_ext library
-# Copyright 2010-2013 Jonas Kubilius
+# Copyright 2010-2014 Jonas Kubilius
 # The program is distributed under the terms of the GNU General Public License,
 # either version 3 of the License, or (at your option) any later version.
 
 """Basic command-line and graphic user interface"""
 
-import wx, sys, os, inspect, shutil, subprocess
+import sys, os, inspect, shutil, subprocess
 from types import ModuleType
 
 try:
-    from agw import advancedsplash as AS
-except ImportError: # if it's not there locally, try the wxPython lib.
-    import wx.lib.agw.advancedsplash as AS
+    import wx
+    try:
+        from agw import advancedsplash as AS
+    except ImportError: # if it's not there locally, try the wxPython lib.
+        import wx.lib.agw.advancedsplash as AS
+    has_wx = True
+except:
+    has_wx = False
 
 from psychopy import core
 
@@ -23,10 +26,14 @@ try:
 except:
     from exp import OrderedDict
 
+import report
+
+
 class Control(object):
+
     def __init__(self, exp_choices,
                  title='Project',
-                 size=(550,400)
+                 size=None
                  ):
         """
         Initializes user control interface.
@@ -41,8 +48,10 @@ class Control(object):
         :Kwargs:
             title (str, default: 'Project')
                 Title of the GUI app window.
-            size (tuple of two int, default: (350,400))
-                Size of a GUI app.
+            size (tuple of two int, default: None)
+                Size of a GUI app. If None, tries to fit the contents.
+                However, if you have multiple pages in the Listbook,
+                it will probably do a poor job.
         """
         # Some basic built-in functions
         try:
@@ -59,7 +68,7 @@ class Control(object):
 
         if len(sys.argv) > 1:  # command line interface desired
             if sys.argv[1] == 'report':
-                report(exp_choices, sys.argv)
+                self.report(exp_choices, sys.argv)
             else:
                 self.cmd(exp_choices)
         else:
@@ -74,16 +83,16 @@ class Control(object):
             except:
                 sys.exit('Please provide a message for committing changes')
             else:
-                _repo_action(sys.argv[2:], message=message)
+                _repo_action('commit', message=message)
         elif action == '--register':
             try:
                 tag = sys.argv[2]
             except:
                 sys.exit('Please provide a tag to register')
             else:
-                _repo_action(sys.argv[2:], tag=tag)
+                _repo_action('register', tag=tag)
         elif action == '--push':
-            _repo_action(sys.argv[2:])
+            _repo_action('push')
 
         sys.exit()
 
@@ -91,7 +100,12 @@ class Control(object):
         """
         Heavily stripped-down version of argparse.
         """
-        if len(exp_choices) == 1:  # if just a single choice then just take it
+        # if just a single choice then just take it
+        try:
+            third_is_arg = sys.argv[3].startswith('-')
+        except:
+            third_is_arg = True
+        if third_is_arg and len(exp_choices) == 1:
             input_mod_alias = None
             input_class_alias = sys.argv[1]
             input_func = sys.argv[2]
@@ -102,7 +116,6 @@ class Control(object):
             input_mod_alias = sys.argv[1]
             input_class_alias = sys.argv[2]
             input_func = sys.argv[3]
-            class_order = exp_choices[0].order
             arg_start = 4
             avail_mods = [e.alias for e in exp_choices]
             try:
@@ -110,6 +123,7 @@ class Control(object):
             except:
                 sys.exit("module '%s' not recognized" % input_mod_alias)
             module = exp_choices[idx].module
+            class_order = exp_choices[idx].order
 
         if input_mod_alias is not None:
             if input_mod_alias.startswith('-'):
@@ -197,7 +211,7 @@ class Control(object):
 
                     else:
                         try:
-                            input_value = sys.argv[i+1]
+                            input_value = sys.argv[i+1].lstrip('"').rstrip('"')
                         except IndexError:
                             sys.exit('Expected a value after %s but got nothing'
                                  % input_key)
@@ -260,6 +274,8 @@ class Control(object):
                                 input_func)
 
     def app(self, exp_choices=[], title='Experiment', size=None):
+        if not has_wx:
+            raise Exception('You must have wx to open a psychopy_ext app.')
         app = MyApp()
 
         # initial frame with a gauge on it
@@ -268,7 +284,7 @@ class Control(object):
         panel = wx.Panel(frame)
 
         if len(exp_choices) > 1:
-            lb = Listbook(panel, exp_choices)
+            lb = Listbook(panel, exp_choices, frame)
             # add pages to the listbook
             for num, choice in enumerate(exp_choices):
                 pagepanel = wx.Panel(lb)
@@ -279,7 +295,7 @@ class Control(object):
             panelsizer.Add(booktype, 1,  wx.EXPAND|wx.ALL)
             panel.SetSizer(panelsizer)
         else:  # if there's only one Notebook, don't create a listbook
-            setup_page(exp_choices[0], panel)
+            setup_page(exp_choices[0], panel, frame)
         # nicely size the entire window
         app.splash.Close()
         panel.Fit()
@@ -302,29 +318,30 @@ class Control(object):
 def report(exp_choices, args):
     reports = []
     if len(args) == 2:
-        argnames = [ch[2] for ch in exp_choices]
+        argnames = [ch.alias for ch in exp_choices]
     else:
         argnames = args[2:]
 
     for ch in exp_choices:
-        if ch[2] in argnames:
-            choice = ch[1]
+        if ch.alias in argnames:
+            choice = ch.module
             if isinstance(choice, str):
                 try:
                     __import__(choice)
                 except:
-                    module = None
+                    raise #module = None
                 else:
                     module = sys.modules[choice]
             else:
                 module = choice
             if module is not None:
-                functions = inspect.getmembers(module, inspect.isfunction)
-                for name, func in functions:
+                classes = inspect.getmembers(module, inspect.isclass)
+                for name, cls in classes:
                     if name == 'report':
-                        reports.append((ch[0], func))
+                        reports.append((ch.name, cls))
                         break
-    Report().make(reports)
+    rep = report.Report()
+    rep.make(reports=reports)
 
 def _get_classes(module, input_class_alias=None, class_order=None):
     """
@@ -450,13 +467,22 @@ class StaticBox(wx.StaticBox):
                                            )
             #if len(color): inputLabel.SetForegroundColour(color)
             grid.Add(inputLabel, 1, wx.ALIGN_LEFT)
+
             #create input control
-            if isinstance(initial, bool):
+            if isinstance(initial, bool):  # check box for true/false
                 inputBox = wx.CheckBox(parent, -1)
                 inputBox.SetValue(initial)
-            elif isinstance(initial, int):
-                inputBox = wx.SpinCtrl(parent, size=(60, -1), initial=initial)
-            elif isinstance(initial, tuple):
+            elif isinstance(initial, int): # spin field for ints
+                imin = 0  # wx default
+                imax = 100  # wx default
+                if initial > imax:
+                    imax = initial
+                elif initial < imin:
+                    imin = initial
+
+                inputBox = wx.SpinCtrl(parent, size=(60, -1), initial=initial,
+                                       min=imin, max=imax)
+            elif isinstance(initial, tuple):  # choice for tuples
                 inputBox = wx.Choice(parent, -1,
                             choices=[str(option) for option in initial])
                 ## Somewhat dirty hack that allows us to treat the choice just like
@@ -467,7 +493,7 @@ class StaticBox(wx.StaticBox):
                 #else:
                     #initial = 0
                 inputBox.SetSelection(0)
-            else:
+            else:  # plain text label for eveything else
                 inputLength = wx.Size(max(50, 9*len(unicode(initial))+16), 25)
                 inputBox = wx.TextCtrl(parent,-1,unicode(initial),size=inputLength)
 
@@ -485,11 +511,13 @@ class Page(wx.Panel):
     corresponding to info and rp in :class:`exp.Experiment`, and
     buttons which, when clicked, runs a corresponding method.
     """
-    def __init__(self, parent, class_obj, alias, class_alias):
+    def __init__(self, parent, class_obj, alias, class_alias, frame, pagepanel):
         wx.Panel.__init__(self, parent, -1)
         self.class_obj = class_obj
         self.alias = alias
         self.class_alias = class_alias
+        self.frame = frame
+        self.pagepanel = pagepanel
 
         class_init = class_obj()
         if not hasattr(class_init, 'info'):
@@ -503,13 +531,24 @@ class Page(wx.Panel):
             self.sb2 = StaticBox(self, label="Parameters",
                 content=class_init.rp)
 
+        #collpane = wx.CollapsiblePane(self, wx.ID_ANY, "Details:")#, style=wx.CP_DEFAULT_STYLE|wx.CP_NO_TLW_RESIZE)
+        #collpane.Bind(wx.EVT_COLLAPSIBLEPANE_CHANGED, self.OnPaneChanged)
+        ## now add a test label in the collapsible pane using a sizer to layout it:
+        #win = collpane.GetPane()
+        #paneSz = wx.BoxSizer(wx.VERTICAL)
+        #paneSz.Add(wx.StaticText(win, wx.ID_ANY, "test!"), 1, wx.GROW | wx.ALL, 2)
+        #win.SetSizer(paneSz)
+        #paneSz.SetSizeHints(win)
+        ##collpane.Collapse()
+
         # generate buttons
         # each button launches a function in a given class
         #actions = _get_methods(class_init)
         actions = _get_methods_byname(class_init)
         # buttons will sit on a grid of 2 columns and as many rows as necessary
-        buttons = wx.FlexGridSizer(rows=0, cols=2)
+        buttons_sizer = wx.FlexGridSizer(rows=0, cols=2)
         add = False
+        self.buttons = []
         for i, (label, action) in enumerate(actions):
             if hasattr(class_init, 'actions'):
                 if class_init.actions is not None:
@@ -521,11 +560,13 @@ class Page(wx.Panel):
                 add = True
             if add:
                 run = wx.Button(self, label=label, size=(150, 30))
-                buttons.Add(run, 1)
+                run._proc_running = False
+                buttons_sizer.Add(run, 1)
                 run.info = class_init.info  # when clicked, what to do
                 run.rp = class_init.rp
                 run.action = label
                 run.Bind(wx.EVT_BUTTON, self.OnButtonClick)
+                self.buttons.append(run)
                 if i==0: run.SetFocus()
 
         pagesizer = wx.BoxSizer(wx.VERTICAL)
@@ -535,52 +576,88 @@ class Page(wx.Panel):
         if class_init.rp is not None:
             pagesizer.Add(self.sb2.sizer)
         # put the buttons in the bottom
-        pagesizer.Add(buttons, 1, wx.ALL|wx.ALIGN_LEFT)
+        pagesizer.Add(buttons_sizer, 1, wx.ALL|wx.ALIGN_LEFT)
         self.SetSizer(pagesizer)
 
+        # add the pane with a zero proportion value to the 'sz' sizer which contains it
+        #pagesizer.Add(collpane, 1, wx.GROW | wx.ALL, 5)
+        #pagesizer.Add( collpane, 2, wx.RIGHT|wx.LEFT|wx.EXPAND, 5 )
+
+
     def OnButtonClick(self, event):
-        #module = event.GetEventObject().module
-        ## first update info and rp
-        button = event.GetEventObject()
-        if button.info is not None:
-            for key, field in zip(button.info.keys(), self.sb1.inputFields):
-                button.info[key] = field.GetValue()
-        if button.rp is not None:
-            for key, field in zip(button.rp.keys(), self.sb2.inputFields):
-                button.rp[key] = field.GetValue()
+            button = event.GetEventObject()
+        #if button._proc_running:
+            #self.enable(button)
+            #self.proc.kill()
+        #else:
+            # if info has tuples, select only one option
+            if button.info is not None and button.info != {}:
+                for key, field in zip(button.info.keys(), self.sb1.inputFields):
+                    button.info[key] = field.GetValue()
+            else:
+                button.info = {}
+            # if info has tuples, select only one option
+            if button.rp is not None:
+                for key, field in zip(button.rp.keys(), self.sb2.inputFields):
+                    button.rp[key] = field.GetValue()
+            else:
+                button.rp = {}
+
+            # call the relevant script
+            opts = [self.alias, self.class_alias, button.GetLabelText()]
+            params = []
+
+            for k,v in button.info.items() + button.rp.items():
+                params.append('--%s' % k)
+                vstr = '%s' % v
+                if len(vstr.split(' ')) > 1:
+                    vstr = '"%s"' % vstr
+                params.append(vstr)
+            command = [sys.executable, sys.argv[0]] + opts + params
+
+            #button._origlabel = opts[2]
+            #button.SetLabel('kill')
+            #button._proc_running = True
+            button.proc = subprocess.Popen(command, shell=False)  # no shell is safer
+
+            #if button.proc.poll() is not None:  # done yet?
+                #self.enable(button)
+
+    def enable(self, button):
+        button.SetLabel(button._origlabel)
+        button._proc_running = False
+
+    #def OnPaneChanged(self, evt):
+        ## redo the layout
+        #self.GetSizer().Layout()
+        #self.Fit()
+        #self.pagepanel.GetSizer().Layout()
+        #self.pagepanel.Layout()
+        #self.pagepanel.Fit()
+        #self.frame.Layout()
+        #self.frame.Fit()
 
-        # call the relevant script
-        opts = [self.alias, self.class_alias, button.GetLabelText()]
-        params = []
-        for k,v in button.info.items() + button.rp.items():
-            params.append('--%s' % k)
-            vstr = '%s' % v
-            if len(vstr.split(' ')) > 1:
-                vstr = '"%s"' % vstr
-            params.append(vstr)
-        print params
-        command = [sys.executable, sys.argv[0]] + opts + params
-        subprocess.call(command, shell=False)  # no shell is safer
 
 
 class Listbook(wx.Listbook):
     """
     Listbook class
     """
-    def __init__(self, parent, exp_choices):
+    def __init__(self, parent, exp_choices, frame):
         wx.Listbook.__init__(self, parent, id=wx.ID_ANY)
         self.exp_choices = exp_choices
         self.ready = []
         self.Bind(wx.EVT_LISTBOOK_PAGE_CHANGING, self.OnPageChanging)
+        self.frame = frame
 
     def OnPageChanging(self, event):
         new = event.GetSelection()
         if new not in self.ready:
-            success = setup_page(self.exp_choices[new], self.GetPage(new))
+            success = setup_page(self.exp_choices[new], self.GetPage(new), self.frame)
             if success:
                 self.ready.append(new)
 
-def setup_page(choice, pagepanel):
+def setup_page(choice, pagepanel, frame):
     """
     Creates a :class:`Page` inside a :class:`Notebook`.
 
@@ -602,8 +679,10 @@ def setup_page(choice, pagepanel):
         class_aliases, class_obj = _get_classes(choice.module, class_order=choice.order)
 
     nb = wx.Notebook(pagepanel)
+
+
     for class_alias, class_obj in class_aliases:
-        nb.AddPage(Page(nb, class_obj, choice.alias, class_alias), class_alias)
+        nb.AddPage(Page(nb, class_obj, choice.alias, class_alias, frame, pagepanel), class_alias)
     panelsizer = wx.BoxSizer()
     panelsizer.Add(nb, 1,  wx.EXPAND|wx.ALL)
     pagepanel.SetSizer(panelsizer)
@@ -637,6 +716,16 @@ def _repo_action(cmd, **kwargs):
     'Registration' is inspired by the `Open Science Framework
     <http://openscienceframework.org/>`_. Useful when you start running
     participants so that you can always go back to that version.
+
+    To set up ``git`` on Windows, you should download it from the
+    ``git-scm.com`` website, install it choosing to add git to the registry
+    such that it could be used from cmd too (Option 2),
+    use their Git Bash to generate an SSH key
+    (`follow these instructions <https://help.github.com/articles/generating-ssh-keys>`_),
+    and create a ``HOME`` variable in your environment that hold a path
+    to your user on Windows (because in it there is .ssh folder where you
+    saved your SSH key). This ``HOME`` variable is used by git to know
+    where to look for keys.
     """
     rev = _detect_rev()
     if rev == 'hg':
@@ -660,7 +749,7 @@ def _repo_action(cmd, **kwargs):
             call = 'git push'
         elif cmd == 'commit':
             if 'message' in kwargs:
-                call = 'git commit -am "%s"' % kwargs['message']
+                call = ['git add -A', 'git commit -m "%s"' % kwargs['message']]
             else:
                 raise Exception('Please provide a message for committing changes')
         else:
@@ -671,15 +760,18 @@ def _repo_action(cmd, **kwargs):
     else:
         raise Exception("%s is not supported for %s yet" % (rev, cmd))
 
-    out, err = core.shellCall(call, stderr=True)
-    call = '$ ' + call
-    write = [call]
-    if out != '':
-        write.append(out)
-    if err != '':
-        write.append(err)
-    sys.stdout.write('\n'.join(write) + '\n')
-    return call, out, err
+    if not isinstance(call, (list, tuple)):
+        call = [call]
+    tmp_out = ''
+    for c in call:
+        out, err = core.shellCall(c, stderr=True)
+        tmp_out += '\n'.join([out, err])
+    output = ['$ ' + c for c in call]
+    if tmp_out != '':
+        output.append(tmp_out)
+    output = '\n'.join(output) + '\n'
+    sys.stdout.write(output)
+    return output
 
 
 class Choices(object):
@@ -742,79 +834,70 @@ class Choices(object):
         self.order = order
 
 
-class Report(object):
+class Arg(dict):
+    def __init__(self, key, value, advanced=True, label=None, guess=False):
+        self.dict = {#key: key,
+                     #value: value,
+                     'advanced': advanced,
+                     'label': label,
+                     'guess': guess}
+        super(Arg, self).__init__(self.dict)
+        self.__dict__ = self
+        self.key = key
+        self.value = value
 
-    def __init__(self, output='html', path='report'):
-        self.path = path+'/'
-        #if not os.path.isdir(self.paths['report']):
-            #os.makedirs(self.paths['report'])
-        #self.rp['plot'] = False
-        #self.rp['saveplot'] = True
-        #self.rp['html'] = True
+class Params(OrderedDict):
 
-    def write(self, text):
-        self.htmlfile.write(text)
+    def __init__(self, params):
 
-    def make(self, reports):
-        if not os.path.isdir(self.path):
-            os.makedirs(self.path)
+        #for p in params:
+            #self.update({p.key: p})
+            #self[p.key] = p
+
+        if isinstance(params, (dict, OrderedDict)):
+            params = [Arg(k,v) for k,v in params.items()]
+            #if isinstance(p, (dict, OrderedDict)):
+                #p = Arg(p)
+
+        dict_vals = [(p.key, p) for p in params]
+        super(Params, self).__init__(dict_vals)
+        #import pdb; pdb.set_trace()
+        self.params = params
+
+        #self.update(dict_vals)
+
+        #self.__dict__ = self
+
+    def __getitem__(self, key):
+        return super(Params, self).__getitem__(key).value
+
+    def __getattr__(self, key):
+        if key in self:
+            return super(Params, self).__getitem__(key)
         else:
-            for root, dirs, files in os.walk(self.path):
-                #for f in files:
-                    #try:
-                          #os.unlink(os.path.join(root, f))
-                    #except:
-                        #pass
-                for d in dirs:
-                    try:
-                          shutil.rmtree(os.path.join(root, d))
-                    except:
-                        pass
+            return super(Params, self).__getattr__(key)
 
-        src = os.path.dirname(__file__)
-        src = os.path.join(src, 'resources/')
-        import glob
-        for f in glob.glob(src+'*'):
-            if os.path.isfile(f):
-                if os.path.basename(f) != 'index.html':
-                    shutil.copy2(f)
+    #def __setitem__(self, key, value):
+        #super(Params,self).__setitem__(key, value)
+
+    def update(self, new_dict):
+        #import pdb; pdb.set_trace()
+        #if isinstance(new_dict, (OrderedDict, dict)):
+            ##import pdb; pdb.set_trace()
+            #new_dict = Params(new_dict)
+        for key in new_dict:
+            try:
+                new_item = getattr(new_dict, key)
+            except:
+                new_item = Arg(key, new_dict[key])
+                #import pdb; pdb.set_trace()
+
+            if key in self:
+                #
+                #super(Params, self).update(
+                item = getattr(self, key)
+                #import pdb; pdb.set_trace()
+                item.update(new_item)
             else:
-                dst = os.path.join(self.path, os.path.basename(f))
-                shutil.copytree(f, dst,
-                    ignore=shutil.ignore_patterns('index.html'))
-        with open(src + 'index.html', 'rb') as tmp:
-            template = tmp.read().split('####REPLACE####')
-        self.htmlfile = open(self.path + 'index.html', 'wb')
-        self.write(template[0])
-        for name, report in reports:
-            self.writeh(name, h='h1')
-            report(self)
-
-        self.write(template[1])
-        self.htmlfile.close()
-
-    def writeimg(self, names, caption=None, plt=None, win=None):
-        if isinstance(names, str):
-            names = [names]
-        fname = '_'.join(names) + '.png'
-        imgdir = self.path + self.imgpath
-        if plt is not None or win is not None:
-            if not os.path.isdir(imgdir):
-                os.makedirs(imgdir)
-            if plt is not None:
-                plt.savefig(imgdir + fname, bbox_inches='tight')
-            elif win is not None:
-                win.saveMovieFrames(imgdir + fname)
-        if caption is None:
-            caption = ' '.join(names)
-        self.htmlfile.write(
-            '<figure>\n'
-            '    <img src="%s" />\n'
-            '    <figcaption><strong>Figure.</strong> %s</figcaption>\n'
-            '</figure>\n' % (self.imgpath + fname, caption)
-            )
-
-    def writeh(self, text, h='h1'):
-        self.htmlfile.write('<%s>%s</%s>\n' % (h, text, h))
-
-
+                #import pdb; pdb.set_trace()
+                self[key] = new_item
